/' ==================================================================================================== '/
/' FMOD Ex - output development header file. Copyright (c), Firelight Technologies Pty, Ltd. 2004-2011. '/
/'                                                                                                      '/
/' Use this header if you are wanting to develop your own output plugin to use with                     '/
/' FMOD's output system.  With this header you can make your own output plugin that FMOD                '/
/' can register and use.  See the documentation and examples on how to make a working plugin.           '/
/'                                                                                                      '/
/' ==================================================================================================== '/

#ifndef _FMOD_OUTPUT_BI
#define _FMOD_OUTPUT_BI

#include "fmodex.bi"

type as FMOD_OUTPUT_STATE_ FMOD_OUTPUT_STATE

/'
    Output callbacks
'/ 
type FMOD_OUTPUT_GETNUMDRIVERSCALLBACK as function(byval output_state as FMOD_OUTPUT_STATE ptr, byval numdrivers as integer ptr) as FMOD_RESULT
type FMOD_OUTPUT_GETDRIVERNAMECALLBACK as function(byval output_state as FMOD_OUTPUT_STATE ptr, byval id as integer, byval name_ as zstring ptr, _
byval namelen as integer) as FMOD_RESULT
type FMOD_OUTPUT_GETDRIVERCAPSCALLBACK as function(byval output_state as FMOD_OUTPUT_STATE ptr, byval id as integer, _
byval caps as FMOD_CAPS ptr) as FMOD_RESULT
type FMOD_OUTPUT_INITCALLBACK  as function(byval output_state as FMOD_OUTPUT_STATE ptr, byval selecteddriver as integer, _
byval flags as FMOD_INITFLAGS, byval outputrate as integer ptr, byval outputchannels as integer, byval outputformat as FMOD_SOUND_FORMAT ptr, _
byval dspbufferlength as integer, byval dspnumbuffers as integer, byval extradriverdata as any ptr) as FMOD_RESULT
type FMOD_OUTPUT_CLOSECALLBACK as function(byval output_state as FMOD_OUTPUT_STATE ptr) as FMOD_RESULT
type FMOD_OUTPUT_UPDATECALLBACK as function(byval output_state as FMOD_OUTPUT_STATE ptr) as FMOD_RESULT
type FMOD_OUTPUT_GETHANDLECALLBACK as function(byval output_state as FMOD_OUTPUT_STATE ptr, byval handle as any ptr ptr) as FMOD_RESULT
type FMOD_OUTPUT_GETPOSITIONCALLBACK as function(byval output_state as FMOD_OUTPUT_STATE ptr, byval pcm as uinteger ptr) as FMOD_RESULT
type FMOD_OUTPUT_LOCKCALLBACK as function(byval output_state as FMOD_OUTPUT_STATE ptr, byval offset as uinteger, byval length as uinteger, _
byval ptr1 as any ptr ptr, byval ptr2 as any ptr ptr, byval len1 as uinteger ptr, byval len2 as uinteger ptr) as FMOD_RESULT
type FMOD_OUTPUT_UNLOCKCALLBACK as function(byval output_state as FMOD_OUTPUT_STATE ptr, byval ptr1 as any ptr, byval ptr2 as any ptr, _
byval len1 as uinteger, byval len2 as uinteger) as FMOD_RESULT
type FMOD_OUTPUT_READFROMMIXER as function(byval output_state as FMOD_OUTPUT_STATE ptr, byval buffer as any ptr, byval length as uinteger) as FMOD_RESULT


/'
[STRUCTURE] 
[
    [DESCRIPTION]
    When creating an output, declare one of these and provide the relevant callbacks and name for FMOD to use when it opens and reads a file of this type.

    [REMARKS]
    Members marked with [in] mean the variable can be written to.  The user can set the value.
    Members marked with [out] mean the variable is modified by FMOD and is for reading purposes only.  Do not change this value.

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation Portable, PlayStation 3, Wii, iPhone, 3GS, NGP, Android

    [SEE_ALSO]
    FMOD_OUTPUT_STATE
]
'/
type FMOD_OUTPUT_DESCRIPTION
    as const zstring ptr                        name                  /' [in] Name of the output. '/
    as uinteger                       version               /' [in] Plugin writer's version number. '/
    as integer                                polling               /' [in] If TRUE (non zero), this tells FMOD to start a thread and call getposition / lock / unlock for feeding data.  If 0, the output is probably callback based, so all the plugin needs to do is call readfrommixer to the appropriate pointer. '/ 
    as FMOD_OUTPUT_GETNUMDRIVERSCALLBACK  getnumdrivers         /' [in] For sound device enumeration.  This callback is to give System::getNumDrivers somthing to return. '/
    as FMOD_OUTPUT_GETDRIVERNAMECALLBACK  getdrivername         /' [in] For sound device enumeration.  This callback is to give System::getDriverName somthing to return. '/
    as FMOD_OUTPUT_GETDRIVERCAPSCALLBACK  getdrivercaps         /' [in] For sound device enumeration.  This callback is to give System::getDriverCaps somthing to return. '/
    as FMOD_OUTPUT_INITCALLBACK           init                  /' [in] Initialization function for the output device.  This is called from System::init. '/
    as FMOD_OUTPUT_CLOSECALLBACK          close                 /' [in] Cleanup / close down function for the output device.  This is called from System::close. '/
    as FMOD_OUTPUT_UPDATECALLBACK         update                /' [in] Update function that is called once a frame by the user.  This is called from System::update. '/
    as FMOD_OUTPUT_GETHANDLECALLBACK      gethandle             /' [in] This is called from System::getOutputHandle.  This is just to return a pointer to the internal system device object that the system may be using.'/
    as FMOD_OUTPUT_GETPOSITIONCALLBACK    getposition           /' [in] This is called from the FMOD software mixer thread if 'polling' = true.  This returns a position value in samples so that FMOD knows where and when to fill its buffer. '/
    as FMOD_OUTPUT_LOCKCALLBACK           lock                  /' [in] This is called from the FMOD software mixer thread if 'polling' = true.  This function provides a pointer to data that FMOD can write to when software mixing. '/
    as FMOD_OUTPUT_UNLOCKCALLBACK         unlock                /' [in] This is called from the FMOD software mixer thread if 'polling' = true.  This optional function accepts the data that has been mixed and copies it or does whatever it needs to before sending it to the hardware. '/
end type


/'
[STRUCTURE] 
[
    [DESCRIPTION]
    Output plugin structure that is passed into each callback.

    [REMARKS]
    Members marked with [in] mean the variable can be written to.  The user can set the value.
    Members marked with [out] mean the variable is modified by FMOD and is for reading purposes only.  Do not change this value.

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation Portable, PlayStation 3, Wii, iPhone, 3GS, NGP, Android

    [SEE_ALSO]
    FMOD_OUTPUT_DESCRIPTION
]
'/
type FMOD_OUTPUT_STATE_
    as any ptr                      plugindata      /' [in] Plugin writer created data the output author wants to attach to this object. '/
    as FMOD_OUTPUT_READFROMMIXER  readfrommixer   /' [out] Function to update mixer and write the result to the provided pointer.  Used from callback based output only.  Polling based output uses lock/unlock/getposition. '/
end type

#endif
